#!/usr/bin/tclsh

################################################################################
#  1.0  Description
#
#  tcl script for generating OOo toolbar/menu bar themes from kde icon themes
################################################################################

# begin   10/22/03
# author  Rohit Kaul
# email   rohitkaul@sbcglobal.net

# Version 0.1
# Version 0.12 (10/25/03)

# License
# -------
# GPL
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.

# Warranty
# --------
# Absolutely none! Use at your own risk!


################################################################################
#  2.0 Customizations
#
################################################################################

 set ThemeDir    "MyTheme/22x22/"
 # don't forget the trailing /"
 # ThemeDir should end with the icon-size (e.g. 22x22)
 # dump the 22x22 icons (actions, apps, etc) all in this directory

 set dx 1
 set dy 1
 # offset of 0 pixel is min., default should be okay

 set example_background_color "#f1f1f1"; # light
#  set example_background_color "#C0C0C0"; # gray
 set bg $example_background_color

 global maskImg;
 set maskImg "false"; # true/false
 # masking doesn't go well with elaborate icons, but
 # may look okay for sharp edged icons

################################################################################
#  3. Procs
################################################################################


proc isValidNum { num } {

	set val -1
	scan $num %d val

    if { $val >= 0 && $val < 1000 } {
    	return 1
    }
		return 0
}


proc isValid { num name } {
	upvar BaseDir BaseDir
	upvar ThemeDir ThemeDir

	set res [isValidNum $num]

    if { $res == 0 } {
    	return 0;
    } else {

    	set res 0;
		set pName $name
        set filePath $ThemeDir$pName

        if { [file isfile $filePath] } {
        	set res 1
        }

        set pName p$num.png
        set filePath $BaseDir$pName
        if { [file isfile $filePath] } {
        	set res 1
        }
		return $res;
    }
}


proc simplifyLine { line splitChar } {

	set  simLine ""
	set  spChar  " "
	set  tmpl   [string trim $line $splitChar]
	set  tmpl   [split $tmpl $splitChar]

	set count 0
	foreach seq $tmpl {
		set  seq  [lindex $tmpl $count]
		set  seq  [string trim $seq]
		set  simLine $simLine$seq$spChar
		incr count
	}
	return $simLine
}



proc createImageXML { } {
	upvar ImageXml  ImageXml
	upvar SlotFile  SlotFile
	upvar BitMapDir BitMapDir
	upvar bg        bg
	upvar ThemeDir  ThemeDir
    upvar BaseDir   BaseDir

    global maskImg
	set maskColor $bg;

	set xmlFile $ImageXml

	# first the header
	set fp    [open $xmlFile  w]

	puts $fp "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
	puts $fp "<!DOCTYPE image:imagecontainer PUBLIC \"-//OpenOffice.org//DTD OfficeDocument 1.0//EN\" \"image.dtd\">"
	puts $fp "<image:imagescontainer xmlns:image=\"http://openoffice.org/2001/image\""
	puts $fp "xmlns:xlink=\"http://www.w3.org/1999/xlink\">"

	if { $maskImg == "true"} {
	puts $fp "<image:images xlink:type=\"simple\" xlink:href=\"$BitMapDir/userimages.bmp\" image:maskmode=\"maskbitmap\" image:maskurl=\"$BitMapDir/userimagesmask.bmp\">"
	} else {
	puts $fp "<image:images xlink:type=\"simple\" xlink:href=\"$BitMapDir/userimages.bmp\" image:maskmode=\"maskcolor\" image:maskcolor=\"$maskColor\">"
	}


	set op [open $SlotFile r]
	set lst [read $op]
	close $op

	set inc 0
	set lst [split $lst "\n"]

	foreach icon $lst {
	set icon  [simplifyLine $icon ","]
	set icon  [split $icon]

	set pNum  [lindex $icon 0]
	set pName [lindex $icon 1]
	set pSlot [lindex $icon 2]

      if { [isValid $pNum $pName] } {
		puts $fp "<image:entry image:bitmap-index=\"$inc\" image:command=\"$pSlot\"/>"

		incr inc
		}
	}
	puts $fp "</image:images>"
    puts $fp "</image:imagescontainer>"

	close $fp
}

proc nextImg {inc pNum pName delx} {
upvar x0    x0
upvar y0    y0
upvar out1  out1
upvar out2  out2
upvar example example
upvar cx    cx
upvar cy    cy
upvar dx    dx
upvar dy    dy
upvar bg    bg
upvar BitMapDir  BitMapDir
upvar BaseDir    BaseDir
upvar ThemeDir   ThemeDir
upvar themeIcons themeIcons
global maskImg

	set pNum   "p$pNum"
	set pNumfl "$pNum.png"
	set img    "$BaseDir$pNumfl"

	set custom 0

	if { [lsearch $themeIcons $pName] != -1 } {
		set img $ThemeDir$pName
		set custom 1
	}

	if { [lsearch $themeIcons $pNumfl] != -1 } {
		set img $ThemeDir$pNumfl
		set custom 1
	}

	exec composite -geometry +$x0+$y0 $img $out1 $out1
	exec composite -geometry +$x0+$y0 $img $example $example

    if { $maskImg == "true" } {
		set imgMsk $img
		exec composite -geometry +$x0+$y0 $imgMsk $out2 $out2
    }

	if { $custom == 0 } {
		puts -nonewline "."
	} else {
		puts -nonewline "*"
	}

	flush stdout
}




proc cleanup { } {
	exec rm -f lst.txt
    exec rm -f acts.txt
    exec rm -f apps.txt
}


################################################################################
# 4. Initialize
################################################################################
set SlotFile   "slotlist.csv"
set ImageXml   "imagelist.xml"
set BitMapDir  "genBitmaps"
set numIcons   0


    puts "ThemeDir: $ThemeDir"
    set sz [split $ThemeDir "/"]
	set sz  [lindex $sz [expr {[llength $sz] -2}]]

	scan $sz %d cx
	scan $sz %d cy

	puts "icon-size = ($cx,$cy)"
	set BaseDir     ".BaseIcons/22x22/"

    if { $cy < 19 } {
    set BaseDir     ".BaseIcons/16x16/"
    }

	set op [open $SlotFile r]
	set lst [read $op]
	close $op

	set lst [split $lst "\n"]
	foreach icon $lst {
    	set icon  [simplifyLine $icon ","]
		set  pNum  [lindex $icon 0];
        set  pName [lindex $icon 1];
        if { [isValid $pNum $pName] } {
			incr numIcons
		}
	}

	puts "num icons: $numIcons"

set out1    "$BitMapDir/userimages.png"
set example "$BitMapDir/userimages.bmp"
set out2    "$BitMapDir/userimagesmask.bmp"
set OObitmapDir $BitMapDir


	exec ls -w 1 $ThemeDir > lst.txt
	set fp [open lst.txt r]
	set themeIcons [read $fp]
	close $fp
	set themeIcons [split $themeIcons "\n"]


	set cx [expr {$cx + 2*$dx}]
	set cy [expr {$cy + 2*$dy}]

	set x0 $dx
	set y0 $dy

	set len [expr {$numIcons*$cx}]


################################################################################
#  5. Generate tool-bar
################################################################################

	createImageXML
	exec rm -Rf $BitMapDir
	exec mkdir  $BitMapDir

	set x "x"
	exec convert -size $len$x$cy xc:transparent    $out1
    exec convert -size $len$x$cy xc:$bg            $example
    if { $maskImg == "true" } {
	exec convert -size $len$x$cy xc:white          $out2
    }

	# start iterations
	set op [open $SlotFile r]
	set lst [read $op]
	close $op

	set inc 0
	set lst [split $lst "\n"]



	foreach icon $lst {
		set icon  [simplifyLine $icon ","]
		set icon  [split $icon]

		set pNum  [lindex $icon 0]
		set pName [lindex $icon 1]
		set pSlot [lindex $icon 2]


        if { [isValid $pNum $pName] } {
			nextImg $inc $pNum $pName $cx
			incr inc 1
			set x0 [expr {$x0 + $cx}]
		}
	}

	puts " done"
	cleanup

